package net.monarchpass.piecannon;
 
import android.app.Activity;
import android.os.Bundle;
import android.content.Intent;
import android.net.Uri;
import android.widget.TextView;
import android.content.ContentResolver;

import com.google.common.io.ByteSource;

import java.net.URI;
import java.io.File;
import java.io.InputStream;
import java.io.IOException;
import java.io.StringWriter;
import java.io.PrintWriter;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ExecutionException;

import jodd.net.MimeTypes;

import lombok.AllArgsConstructor;

public class Send extends Activity {
    private final ExecutorService executor = Executors.newFixedThreadPool(1);

    @Override
    public void onCreate (final Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_send);
        
        final Intent intent = getIntent();
        if (intent.hasExtra (Intent.EXTRA_STREAM)) {
            final Uri imageUri = intent.getParcelableExtra(Intent.EXTRA_STREAM);
            if (imageUri != null) {
                uploadImage(imageUri);
            }
        }
    }
    
    public void uploadImage (final Uri imageUri) {
        // handle image
        final TextView text = (TextView)findViewById(R.id.text);
        text.append("\n---\nReceived file url: \n");
        text.append(imageUri.toString());
        
        final ContentResolver resolver = getContentResolver();
        final PieCannon cannon = new PieCannon();
        
        try
        {
            cannon.loadServersFrom(new File(getExternalFilesDir(null), "servers.json"));
        }
        catch (final Exception exception)
        {
            logException(exception);
            return;
        }
        
        executor.submit(() -> {
            try
            {
                final URI uploaded = cannon.selectServer().upload(createFileNameForUri(imageUri, resolver), new ContentByteSource(resolver, imageUri));
                runOnUiThread(() -> {
                    text.append("\n---\nUploaded file to url: \n");
                    text.append(uploaded.toString());
                    shareUploadUrl(uploaded);                
                });
            }
            catch (final Exception exception) {
                logException(exception);
            }
        });
    }
    
    public void shareUploadUrl (final URI uploadedUrl) {
        final Intent sharingIntent = new Intent(Intent.ACTION_SEND);
        sharingIntent.setType("text/plain");
        sharingIntent.putExtra(android.content.Intent.EXTRA_TEXT, uploadedUrl.toString());
        startActivity(Intent.createChooser(sharingIntent, "Share uploaded url"));
    }
    
    public static String createFileNameForUri (final Uri uri, final ContentResolver resolver) {
        final String filename = uri.getLastPathSegment();
        if (filename.contains(".")) {
            return filename;
        }
        
        final String type = resolver.getType(uri);
        final String[] extensions = MimeTypes.findExtensionsByMimeTypes(type, false);
        if (extensions.length > 0) {
            return filename + "." + extensions[0];        
        }
        
        throw new RuntimeException("Could not construct a file name");
    }
    
    private void logException (final Exception exception) {
        final StringWriter buf = new StringWriter();
        final PrintWriter out = new PrintWriter(buf);
        exception.printStackTrace(out);
        runOnUiThread(() -> {
            final TextView text = (TextView)findViewById(R.id.text);
            text.append("\n---\n");
            text.append(buf.toString());
        });
    }
    
    @AllArgsConstructor
    public static class ContentByteSource extends ByteSource {
        private final ContentResolver resolver;
        private final Uri contentUri;
    
        @Override
        public InputStream openStream () throws IOException
        {
            return resolver.openInputStream(contentUri);
        }
    }
}
