package net.monarchpass.piecannon;

import java.net.URI;

import java.io.File;
import java.util.List;
import org.freedesktop.BaseDirectory;

import lombok.extern.java.Log;
import java.util.logging.Level;

@Log
public class App {
    public static void main (final String... args) throws Exception {
        final File serversJson = getServersJson();
        final PieCannon cannon = new PieCannon();
        final List<Server> servers = cannon.loadServersFrom(serversJson);
        log.log(Level.INFO, "{0} servers loaded from {1}", new Object[] {
            servers.size(), serversJson
        });
        
        if (servers.isEmpty()) {
            log.log(Level.SEVERE, "No servers defined, please define at least one in {0}", serversJson);
            System.exit(1);
        }

        if (args.length == 0) {
            log.log(Level.SEVERE, "No filename provided");
            System.exit(1);
        }
        
        final Server server = cannon.selectServer();
        log.log(Level.INFO, "Randomly selected server: {0}", server.getLabel());
        
        final String arg = args[0];
        if (arg.equals("--test")) {
            testServers(servers);
            return;
        }
        
        final File source = new File(arg);
        if (!source.exists()) {
            log.log(Level.SEVERE, "No such file: {0}", source);
            System.exit(1);
        }
        
        final URI result = server.upload(source);
        System.out.println(result);
    }
 
    public static void testServers (final List<Server> servers) {
        log.log(Level.INFO, "Testing all defined servers");
        for (final Server server : servers) {
            Object result;
            try {
                result = PieCannon.testServer(server);
            }
            catch (final Exception exception) {
                result = exception;
            }
            
            log.log(Level.INFO, "{0} {1}: {2}", new Object[] {
                server.getClass().getSimpleName(), server.getLabel(), result
            });
        }
    }
 
    public static File getServersJson () {
        return new File(getDataDirectory(), "servers.json");
    }
    
    public static File getDataDirectory () {
        return new File(BaseDirectory.get(BaseDirectory.XDG_DATA_HOME), "piecannon");
    }
}
