package net.monarchpass.piecannon;

import java.net.URI;

import java.io.File;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.IOException;

import java.util.List;
import java.util.ArrayList;
import java.util.Random;

import com.google.common.base.Charsets;
import com.google.common.io.CharSource;
import com.google.common.io.ByteStreams;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

import net.monarchpass.piecannon.util.ServerFactory;

public class PieCannon {
    public List<Server> servers;
    
    public List<Server> loadServersFrom (final File serversJson) throws IOException {
        try (final InputStream in = new FileInputStream(serversJson)) {
            return loadServersFrom(in);
        }
    }
    
    public List<Server> loadServersFrom (final InputStream in) throws IOException {
        final ServerFactory factory = new ServerFactory();
        final JsonParser parser = new JsonParser();
        final List<Server> loadedServers = new ArrayList<>();
        for (final JsonElement element : parser.parse(new InputStreamReader(in)).getAsJsonArray()) {
            if (!element.isJsonObject()) {
                continue;
            }
            
            loadedServers.add(factory.apply((JsonObject)element));
        }
        servers = loadedServers;
        return loadedServers;
    }    
    
    public Server selectServer () {
        if (servers == null) {
            throw new IllegalStateException("Servers are not loaded");
        }
        
        return servers.get(new Random().nextInt(servers.size()));    
    }
    
    public static boolean testServer (final Server server) {
        final String testString = "piecannon-test-" + System.currentTimeMillis();
        final URI result = server.upload("piecannon.test", CharSource.wrap(testString).asByteSource(Charsets.UTF_8));
        
        try (final InputStream in = result.toURL().openStream()) {
            return new String(ByteStreams.toByteArray(in), Charsets.UTF_8).equals(testString);
        } catch (final IOException exception) {
            exception.printStackTrace();
        }
        
        return false;
    }
}
