package net.monarchpass.piecannon.impl;

import net.monarchpass.piecannon.Server;
import com.google.common.io.ByteSource;

import java.io.InputStream;
import java.io.IOException;
import java.net.URI;

import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.SftpException;

import lombok.Data;

@Data
public class SftpServer implements Server {
    private final String label;
    private final String host;
    private final int port;
    private final String username;
    private final String password;
    private final String path;
    private final URI uri;
    
    public URI upload (String name, ByteSource source) {
        try {
            final URI target = URI.create(uri.toString() + "/" + name.replace(" ", "%20"));
            
            final JSch jsch = new JSch();
            final Session session = jsch.getSession(username, host, port);
            session.setPassword(password);
            session.setConfig("StrictHostKeyChecking", "no");
            session.connect();

            final Channel channel = session.openChannel("sftp");
            channel.connect();
            
            final ChannelSftp sftp = (ChannelSftp) channel;
            sftp.cd(path);
            
            try (InputStream in = source.openStream()) {
                sftp.put(in, name);
            }
            
            session.disconnect();
            return target;
        } catch (final JSchException | IOException | SftpException exception) {
            throw new RuntimeException(exception);
        }
    }
}
