package net.monarchpass.piecannon.util;

import java.net.URI;
import java.util.Optional;
import java.util.function.Function;

import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;

import net.monarchpass.piecannon.Server;
import net.monarchpass.piecannon.impl.FtpServer;
import net.monarchpass.piecannon.impl.GoFileServer;
import net.monarchpass.piecannon.impl.SftpServer;
import net.monarchpass.piecannon.impl.WebDavServer;

public class ServerFactory implements Function<JsonObject, Server> {
    public Server apply (final JsonObject object) {
        final String type = Optional.ofNullable(object.getAsJsonPrimitive("type"))
            .map(JsonPrimitive::getAsString)
            .orElse("webdav");
        
        if (type.equals("webdav")) {
            return makeWebDavServer(object);
        } else if (type.equalsIgnoreCase("gofile")) {
            return new GoFileServer(Optional.ofNullable(object.getAsJsonPrimitive("label")).map(JsonPrimitive::getAsString).orElse("GoFile"));
        } else if (type.equalsIgnoreCase("ftp")) {
            return makeFtpServer(object);
        } else if (type.equalsIgnoreCase("sftp")) {
            return makeSftpServer(object);
        }
        
        throw new IllegalArgumentException("Invalid server type: " + type); 
    }
    
    private Server makeWebDavServer (final JsonObject object) {
        final String label = Optional.ofNullable(object.getAsJsonPrimitive("label"))
            .map(JsonPrimitive::getAsString)
            .orElse("WebDav");
        final String username = Optional.ofNullable(object.getAsJsonPrimitive("username"))
            .map(JsonPrimitive::getAsString)
            .orElse(null);
        final String password = Optional.ofNullable(object.getAsJsonPrimitive("password"))
            .map(JsonPrimitive::getAsString)
            .orElse(null);
        final String url = object.getAsJsonPrimitive("url").getAsString();
        
        return new WebDavServer(
            label, URI.create(url), username, password
        );
    }    
    
    private Server makeSftpServer (final JsonObject object) {
        final String host = object.getAsJsonPrimitive("host").getAsString();
        final String label = object.has("label") ? object.getAsJsonPrimitive("label").getAsString() : host;
        final int port = object.has("port") ? object.getAsJsonPrimitive("port").getAsInt() : 22;
        final String username = object.getAsJsonPrimitive("username").getAsString();
        final String password = object.getAsJsonPrimitive("password").getAsString();
        final String path = object.getAsJsonPrimitive("path").getAsString();
        final String url = object.getAsJsonPrimitive("url").getAsString();
        
        return new SftpServer(
            label, host, port, username, password,
            path, URI.create(url)
        );
    }
    
    private Server makeFtpServer (final JsonObject object) {
        final String host = object.getAsJsonPrimitive("host").getAsString();
        final String label = object.has("label") ? object.getAsJsonPrimitive("label").getAsString() : host;
        final int port = object.has("port") ? object.getAsJsonPrimitive("port").getAsInt() : 21;
        final String username = object.getAsJsonPrimitive("username").getAsString();
        final String password = object.getAsJsonPrimitive("password").getAsString();
        final String path = object.getAsJsonPrimitive("path").getAsString();
        final String url = object.getAsJsonPrimitive("url").getAsString();
        
        return new FtpServer(
            label, host, port, username, password,
            path, URI.create(url)
        );
    }    
}
